<?php declare(strict_types=1);
require __DIR__ . '/../src/FirstPartyClass.php';
require __DIR__ . '/ThirdPartyClass.php';
require __DIR__ . '/ThirdPartyClassA.php';
require __DIR__ . '/ThirdPartyClassB.php';
