<?php declare(strict_types=1);
/*
 * This file is part of PHPUnit.
 *
 * (c) Sebastian Bergmann <sebastian@phpunit.de>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
function delete_directory(string $directory): void
{
    if (\is_file($directory)) {
        @\unlink($directory);

        return;
    }

    if (!\is_dir($directory)) {
        return;
    }

    foreach (\glob(\rtrim($directory, '/') . '/*') as $path) {
        delete_directory($path);
    }

    @\rmdir($directory);
}
